# -*- coding: utf-8 -*-
from typing import Dict

import matplotlib.pyplot as plt
import mlflow
import numpy as np
import pandas as pd
from rich import print
from sklearn.metrics import explained_variance_score, mean_absolute_error, mean_squared_error, r2_score
from sklearn.model_selection import cross_validate


def score(y_true: pd.DataFrame, y_predict: pd.DataFrame) -> Dict:
    """Calculate the scores of the regression model.

    Parameters
    ----------
    y_true : pd.DataFrame (n_samples, n_components)
        The true target values.

    y_predict : pd.DataFrame (n_samples, n_components)
        The predicted target values.

    Returns
    -------
    scores : dict
        The scores of the regression model.
    """
    mse = mean_squared_error(y_true, y_predict)
    rmse = np.sqrt(mse)
    mae = mean_absolute_error(y_true, y_predict)
    r2 = r2_score(y_true, y_predict)
    evs = explained_variance_score(y_true, y_predict)
    print("Mean Square Error: ", mse)
    print("Root Mean Square Error:", rmse)
    print("Mean Absolute Error:", mae)
    print("R2 Score:", r2)
    print("Explained Variance Score:", evs)
    scores = {
        "Root Mean Square Error": rmse,
        "Mean Absolute Error": mae,
        "R2 Score": r2,
        "Explained Variance Score": evs,
    }
    return scores


def display_cross_validation_scores(scores: np.ndarray, score_name: str) -> Dict:
    """Display the scores of cross-validation.

    Parameters
    ----------
    scores : np.ndarray
        The scores of cross-validation.

    score_name : str
        The name of the score.

    Returns
    -------
    cv_scores : dict
        The scores of cross-validation.
    """
    cv_scores = {
        "Fold Scores": str(scores.tolist()),
        "Mean": scores.mean(),
        "Standard Deviation": scores.std(),
    }
    print("Scores:", cv_scores["Fold Scores"])
    print("Mean:", cv_scores["Mean"])
    print("Standard deviation:", cv_scores["Standard Deviation"])
    mlflow.log_metric(f"CV - {score_name} - Mean", cv_scores["Mean"])
    mlflow.log_metric(f"CV - {score_name} - Standard Deviation", cv_scores["Standard Deviation"])
    return cv_scores


def cross_validation(trained_model: object, X_train: pd.DataFrame, y_train: pd.DataFrame, cv_num: int = 10) -> Dict:
    """Evaluate metric(s) by cross-validation and also record fit/score times.

    Parameters
    ----------
    trained_model : object
        The model trained.

    X_train : pd.DataFrame (n_samples, n_components)
        The training feature data.

    y_train : pd.DataFrame (n_samples, n_components)
        The training target values.

    cv_num : int
        Determines the cross-validation splitting strategy.

    Returns
    -------
    scores_result : dict
        The scores of cross-validation.
    """

    scores = cross_validate(
        trained_model,
        X_train,
        y_train,
        scoring=("neg_root_mean_squared_error", "neg_mean_absolute_error", "r2", "explained_variance"),
        cv=cv_num,
    )
    del scores["fit_time"]
    del scores["score_time"]
    # the keys follow the returns of cross_validate in scikit-learn
    scores2display = {
        "test_neg_root_mean_squared_error": "Root Mean Square Error",
        "test_neg_mean_absolute_error": "Mean Absolute Error",
        "test_r2": "R2 Score",
        "test_explained_variance": "Explained Variance Score",
    }
    scores_result = {"K-Fold": cv_num}
    for key, values in scores.items():
        print("*", scores2display[key], "*")
        if (key == "test_neg_root_mean_squared_error") or (key == "test_neg_mean_absolute_error"):
            cv_scores = display_cross_validation_scores(-values, scores2display[key])
        else:
            cv_scores = display_cross_validation_scores(values, scores2display[key])
        scores_result[scores2display[key]] = cv_scores
        print("-------------")
    return scores_result


def plot_predicted_vs_actual(y_test_predict: pd.DataFrame, y_test: pd.DataFrame, algorithm_name: str) -> None:
    """Plot the testing predict values of the trained model and the testing target values.

    Parameters
    ----------
    y_test_predict : pd.DataFrame (n_samples, n_components)
        The testing predict values.

    y_test : pd.DataFrame (n_samples, n_components)
        The testing target values.

    algorithm_name : str
        The name of the algorithm model.
    """
    plt.scatter(y_test_predict, y_test, color="b")
    plt.plot(y_test_predict, y_test_predict, color="r", linestyle="--", label="Perfect Prediction Line")
    plt.xlabel("Predicted Values")
    plt.ylabel("Actual Values")
    plt.legend()
    plt.title(f"Predicted vs. Actual Diagram - {algorithm_name}")


def plot_residuals(y_test_predict: pd.DataFrame, y_test: pd.DataFrame, algorithm_name: str) -> pd.DataFrame:
    """Plot the residuals of the testing predict values and the testing target values.

    Parameters
    ----------
    y_test_predict : pd.DataFrame (n_samples, n_components)
        The testing predict values.

    y_test : pd.DataFrame (n_samples, n_components)
        The testing target values.

    algorithm_name : str
        The name of the algorithm model.

    Returns
    -------
    residuals : pd.DataFrame (n_samples, n_components)
        The residuals of the testing predict values and the testing target values.
    """
    residuals = y_test_predict.values - y_test.values
    residuals = pd.DataFrame(residuals, columns=["Residuals"])
    plt.scatter(y_test_predict, residuals, color="b")
    plt.axhline(0, color="r", linestyle="--", label="Zero Residual Line")
    plt.title(f"Residuals Diagram - {algorithm_name}")
    plt.xlabel("Predicted Values")
    plt.ylabel("Residuals (Actual - Predicted)'")
    plt.legend()
    return residuals



from sqlalchemy.orm import Session

from .schemas import UserCreate
from .sql_models import User
from .utils import get_password_hash, verify_password


def get_user_by_id(db: Session, user_id: str):
    return db.query(User).filter(User.id == user_id).first()


def get_user_by_username(db: Session, username: str):
    return db.query(User).filter(User.username == username).first()


def get_user_by_email(db: Session, email: str):
    return db.query(User).filter(User.email == email).first()


def get_users(db: Session, skip: int = 0, limit: int = 100):
    return db.query(User).offset(skip).limit(limit).all()


def create_new_user(db: Session, user: UserCreate):
    hashed_password = get_password_hash(user.password)
    db_user = User(username=user.username, email=user.email, hashed_password=hashed_password)
    db.add(db_user)
    db.commit()
    db.refresh(db_user)
    return db_user


def authenticate_user(db, email: str, password: str):
    # user = get_user_by_username(db, username)
    user = get_user_by_email(db, email=email)
    if not user:
        return False
    if not verify_password(password, user.hashed_password):
        return False
    return user



from typing import Dict

from rich import print

from ....constants import SECTION
from ....data.data_readiness import float_input, num_input, str_input


def affinitypropagation_manual_hyper_parameters() -> Dict:
    """Manually set hyperparameters.

    Returns
    -------
    hyper_parameters : dict
    """
    print("damping: The extent to which the current value is maintained relative to incoming values ")
    print("Please specify the number of clusters for AffinityPropagation. A good starting range could be between 0.5 and 1, such as 0.5.")
    damping = float_input(0.5, SECTION[2], "damping: ")
    print("Max Iter: Maximum number of iterations of the algorithm for a single run.")
    print("Please specify the maximum number of iterations of the affinitypropagation algorithm for a single run. A good starting range could be between 100 and 400, such as 200.")
    max_iter = num_input(SECTION[2], "Max Iter: ")
    print("convergence_iter: Number of iterations with no change in the number of estimated clusters that stops the convergence.")
    print("Please specify the convergence number of iterations of the affinitypropagation algorithm. A good starting range could be between 10 and 15, such as 15.")
    convergence_iter = num_input(SECTION[2], "convergence_iter: ")
    print("affinity: Different affinity methods for affinitypropagation in clustering.")
    print("Please specify the affinity to use for the computation. It is generally recommended to leave it as 'euclidean'.")
    affinity = ["euclidean", "precomputed"]
    affinity = str_input(affinity, SECTION[2])
    hyper_parameters = {"damping": damping, "max_iter": max_iter, "convergence_iter": convergence_iter, "affinity": affinity}
    return hyper_parameters



# -*- coding: utf-8 -*-
import os

import pandas as pd
from multipledispatch import dispatch
from rich import print

from ..constants import MLFLOW_ARTIFACT_DATA_PATH, SECTION
from ..data.data_readiness import num_input
from ..model.regression import (
    BayesianRidgeRegression,
    ClassicalLinearRegression,
    DecisionTreeRegression,
    ElasticNetRegression,
    ExtraTreesRegression,
    GradientBoostingRegression,
    KNNRegression,
    LassoRegression,
    MLPRegression,
    PolynomialRegression,
    RandomForestRegression,
    RegressionWorkflowBase,
    RidgeRegression,
    SGDRegression,
    SVMRegression,
    XGBoostRegression,
)
from ._base import ModelSelectionBase


class RegressionModelSelection(ModelSelectionBase):
    """Simulate the normal way of training regression algorithms."""

    def __init__(self, model_name: str) -> None:
        self.model_name = model_name
        self.reg_workflow = RegressionWorkflowBase()
        self.transformer_config = {}

    @dispatch(object, object, object, object, object, object)
    def activate(
        self,
        X: pd.DataFrame,
        y: pd.DataFrame,
        X_train: pd.DataFrame,
        X_test: pd.DataFrame,
        y_train: pd.DataFrame,
        y_test: pd.DataFrame,
    ) -> None:
        """Train by Scikit-learn framework."""

        self.reg_workflow.data_upload(X=X, y=y, X_train=X_train, X_test=X_test, y_train=y_train, y_test=y_test)

        # Model option
        if self.model_name == "Polynomial Regression":
            hyper_parameters = PolynomialRegression.manual_hyper_parameters()
            self.reg_workflow = PolynomialRegression(
                degree=hyper_parameters["degree"],
                interaction_only=hyper_parameters["interaction_only"],
                include_bias=hyper_parameters["include_bias"],
            )
            poly_config, X_train, X_test = self.reg_workflow.poly(X_train, X_test)
            self.transformer_config.update(poly_config)
            self.reg_workflow.data_upload(X_train=X_train, X_test=X_test)
        elif self.model_name == "XGBoost":
            hyper_parameters = XGBoostRegression.manual_hyper_parameters()
            self.reg_workflow = XGBoostRegression(
                n_estimators=hyper_parameters["n_estimators"],
                learning_rate=hyper_parameters["learning_rate"],
                max_depth=hyper_parameters["max_depth"],
                subsample=hyper_parameters["subsample"],
                colsample_bytree=hyper_parameters["colsample_bytree"],
                alpha=hyper_parameters["alpha"],
                lambd=hyper_parameters["lambd"],
            )
        elif self.model_name == "Decision Tree":
            hyper_parameters = DecisionTreeRegression.manual_hyper_parameters()
            self.reg_workflow = DecisionTreeRegression(
                criterion=hyper_parameters["criterion"],
                max_depth=hyper_parameters["max_depth"],
                min_samples_split=hyper_parameters["min_samples_split"],
                min_samples_leaf=hyper_parameters["min_samples_leaf"],
                max_features=hyper_parameters["max_features"],
            )
        elif self.model_name == "Extra-Trees":
            hyper_parameters = ExtraTreesRegression.manual_hyper_parameters()
            self.reg_workflow = ExtraTreesRegression(
                n_estimators=hyper_parameters["n_estimators"],
                max_depth=hyper_parameters["max_depth"],
                min_samples_split=hyper_parameters["min_samples_split"],
                min_samples_leaf=hyper_parameters["min_samples_leaf"],
                max_features=hyper_parameters["max_features"],
                bootstrap=hyper_parameters["bootstrap"],
                oob_score=hyper_parameters["oob_score"],
                max_samples=hyper_parameters["max_samples"],
            )
        elif self.model_name == "Random Forest":
            hyper_parameters = RandomForestRegression.manual_hyper_parameters()
            self.reg_workflow = RandomForestRegression(
                n_estimators=hyper_parameters["n_estimators"],
                max_depth=hyper_parameters["max_depth"],
                min_samples_split=hyper_parameters["min_samples_split"],
                min_samples_leaf=hyper_parameters["min_samples_leaf"],
                max_features=hyper_parameters["max_features"],
                bootstrap=hyper_parameters["bootstrap"],
                oob_score=hyper_parameters["oob_score"],
                max_samples=hyper_parameters["max_samples"],
            )
        elif self.model_name == "Support Vector Machine":
            hyper_parameters = SVMRegression.manual_hyper_parameters()
            self.reg_workflow = SVMRegression(
                kernel=hyper_parameters["kernel"],
                degree=hyper_parameters["degree"],
                gamma=hyper_parameters["gamma"],
                C=hyper_parameters["C"],
                shrinking=hyper_parameters["shrinking"],
            )
        elif self.model_name == "Multi-layer Perceptron":
            hyper_parameters = MLPRegression.manual_hyper_parameters()
            self.reg_workflow = MLPRegression(
                hidden_layer_sizes=hyper_parameters["hidden_layer_sizes"],
                activation=hyper_parameters["activation"],
                solver=hyper_parameters["solver"],
                alpha=hyper_parameters["alpha"],
                learning_rate=hyper_parameters["learning_rate"],
                max_iter=hyper_parameters["max_iter"],
            )
        elif self.model_name == "Linear Regression":
            hyper_parameters = ClassicalLinearRegression.manual_hyper_parameters()
            self.reg_workflow = ClassicalLinearRegression(fit_intercept=hyper_parameters["fit_intercept"])
        elif self.model_name == "K-Nearest Neighbors":
            hyper_parameters = KNNRegression.manual_hyper_parameters()
            self.reg_workflow = KNNRegression(
                n_neighbors=hyper_parameters["n_neighbors"],
                weights=hyper_parameters["weights"],
                algorithm=hyper_parameters["algorithm"],
                leaf_size=hyper_parameters["leaf_size"],
                p=hyper_parameters["p"],
                metric=hyper_parameters["metric"],
            )
        elif self.model_name == "Gradient Boosting":
            hyper_parameters = GradientBoostingRegression.manual_hyper_parameters()
            self.reg_workflow = GradientBoostingRegression(
                n_estimators=hyper_parameters["n_estimators"],
                learning_rate=hyper_parameters["learning_rate"],
                max_depth=hyper_parameters["max_depth"],
                min_samples_split=hyper_parameters["min_samples_split"],
                min_samples_leaf=hyper_parameters["min_samples_leaf"],
                max_features=hyper_parameters["max_features"],
                subsample=hyper_parameters["subsample"],
                loss=hyper_parameters["loss"],
            )
        elif self.model_name == "Lasso Regression":
            hyper_parameters = LassoRegression.manual_hyper_parameters()
            self.reg_workflow = LassoRegression(
                alpha=hyper_parameters["alpha"],
                fit_intercept=hyper_parameters["fit_intercept"],
                max_iter=hyper_parameters["max_iter"],
                tol=hyper_parameters["tol"],
                selection=hyper_parameters["selection"],
            )
        elif self.model_name == "Elastic Net":
            hyper_parameters = ElasticNetRegression.manual_hyper_parameters()
            self.reg_workflow = ElasticNetRegression(
                alpha=hyper_parameters["alpha"],
                l1_ratio=hyper_parameters["l1_ratio"],
                fit_intercept=hyper_parameters["fit_intercept"],
                max_iter=hyper_parameters["max_iter"],
                tol=hyper_parameters["tol"],
                selection=hyper_parameters["selection"],
            )
        elif self.model_name == "SGD Regression":
            hyper_parameters = SGDRegression.manual_hyper_parameters()
            self.reg_workflow = SGDRegression(
                loss=hyper_parameters["loss"],
                penalty=hyper_parameters["penalty"],
                alpha=hyper_parameters["alpha"],
                l1_ratio=hyper_parameters["l1_ratio"],
                fit_intercept=hyper_parameters["fit_intercept"],
                max_iter=hyper_parameters["max_iter"],
                tol=hyper_parameters["tol"],
                shuffle=hyper_parameters["shuffle"],
                learning_rate=hyper_parameters["learning_rate"],
                eta0=hyper_parameters["eta0"],
                power_t=hyper_parameters["power_t"],
            )
        elif self.model_name == "BayesianRidge Regression":
            hyper_parameters = BayesianRidgeRegression.manual_hyper_parameters()
            self.reg_workflow = BayesianRidgeRegression(
                tol=hyper_parameters["tol"],
                alpha_1=hyper_parameters["alpha_1"],
                alpha_2=hyper_parameters["alpha_2"],
                lambda_1=hyper_parameters["lambda_1"],
                lambda_2=hyper_parameters["lambda_2"],
                alpha_init=hyper_parameters["alpha_init"],
                lambda_init=hyper_parameters["lambda_init"],
                compute_score=hyper_parameters["compute_score"],
                fit_intercept=hyper_parameters["fit_intercept"],
                copy_X=hyper_parameters["copy_X"],
                verbose=hyper_parameters["verbose"],
            )
        elif self.model_name == "Ridge Regression":
            hyper_parameters = RidgeRegression.manual_hyper_parameters()
            self.reg_workflow = RidgeRegression(
                alpha=hyper_parameters["alpha"],
                fit_intercept=hyper_parameters["fit_intercept"],
                max_iter=hyper_parameters["max_iter"],
                tol=hyper_parameters["tol"],
            )

        self.reg_workflow.show_info()

        # Use Scikit-learn style API to process input data
        self.reg_workflow.fit(X_train, y_train)
        y_train_predict = self.reg_workflow.predict(X_train)
        y_train_predict = self.reg_workflow.np2pd(y_train_predict, y_train.columns)
        self.reg_workflow.data_upload(y_train_predict=y_train_predict)
        y_test_predict = self.reg_workflow.predict(X_test)
        y_test_predict = self.reg_workflow.np2pd(y_test_predict, y_test.columns)
        self.reg_workflow.data_upload(y_test_predict=y_test_predict)

        # Save the model hyper-parameters
        self.reg_workflow.save_hyper_parameters(hyper_parameters, self.model_name, os.getenv("GEOPI_OUTPUT_PARAMETERS_PATH"))

        # Common components for every regression algorithm
        self.reg_workflow.common_components()

        # Special components of different algorithms
        self.reg_workflow.special_components()

        # Save the prediction result
        self.reg_workflow.data_save(y_train_predict, "Y Train Predict", os.getenv("GEOPI_OUTPUT_ARTIFACTS_DATA_PATH"), MLFLOW_ARTIFACT_DATA_PATH, "Model Train Prediction")
        self.reg_workflow.data_save(y_test_predict, "Y Test Predict", os.getenv("GEOPI_OUTPUT_ARTIFACTS_DATA_PATH"), MLFLOW_ARTIFACT_DATA_PATH, "Model Test Prediction")

        # Save the trained model
        self.reg_workflow.model_save()

    @dispatch(object, object, object, object, object, object, bool)
    def activate(
        self,
        X: pd.DataFrame,
        y: pd.DataFrame,
        X_train: pd.DataFrame,
        X_test: pd.DataFrame,
        y_train: pd.DataFrame,
        y_test: pd.DataFrame,
        is_automl: bool,
    ) -> None:
        """Train by FLAML framework + RAY framework."""

        self.reg_workflow.data_upload(X=X, y=y, X_train=X_train, X_test=X_test, y_train=y_train, y_test=y_test)

        # Model option
        if self.model_name == "Polynomial Regression":
            print("Please specify the maximal degree of the polynomial features.")
            poly_degree = num_input(SECTION[2], "@Degree:")
            self.reg_workflow = PolynomialRegression(degree=poly_degree)
            poly_config, X_train, X_test = self.reg_workflow.poly(X_train, X_test)
            self.transformer_config.update(poly_config)
            self.reg_workflow.data_upload(X_train=X_train, X_test=X_test)
        elif self.model_name == "XGBoost":
            self.reg_workflow = XGBoostRegression()
        elif self.model_name == "Decision Tree":
            self.reg_workflow = DecisionTreeRegression()
        elif self.model_name == "Extra-Trees":
            self.reg_workflow = ExtraTreesRegression()
        elif self.model_name == "Random Forest":
            self.reg_workflow = RandomForestRegression()
        elif self.model_name == "Support Vector Machine":
            self.reg_workflow = SVMRegression()
        elif self.model_name == "Multi-layer Perceptron":
            self.reg_workflow = MLPRegression()
        elif self.model_name == "Linear Regression":
            self.reg_workflow = ClassicalLinearRegression()
        elif self.model_name == "K-Nearest Neighbors":
            self.reg_workflow = KNNRegression()
        elif self.model_name == "Gradient Boosting":
            self.reg_workflow = GradientBoostingRegression()
        elif self.model_name == "Lasso Regression":
            self.reg_workflow = LassoRegression()
        elif self.model_name == "Elastic Net":
            self.reg_workflow = ElasticNetRegression()
        elif self.model_name == "SGD Regression":
            self.reg_workflow = SGDRegression()
        elif self.model_name == "BayesianRidge Regression":
            self.reg_workflow = BayesianRidgeRegression()
        elif self.model_name == "Ridge Regression":
            self.reg_workflow = RidgeRegression()

        self.reg_workflow.show_info()

        # Use Scikit-learn style API to process input data
        self.reg_workflow.fit(X_train, y_train, is_automl)
        y_train_predict = self.reg_workflow.predict(X_train, is_automl)
        y_train_predict = self.reg_workflow.np2pd(y_train_predict, y_train.columns)
        self.reg_workflow.data_upload(y_train_predict=y_train_predict)
        y_test_predict = self.reg_workflow.predict(X_test, is_automl)
        y_test_predict = self.reg_workflow.np2pd(y_test_predict, y_test.columns)
        self.reg_workflow.data_upload(y_test_predict=y_test_predict)

        # Save the model hyper-parameters
        if self.reg_workflow.ray_best_model is not None:
            self.reg_workflow.save_hyper_parameters(self.reg_workflow.ray_best_model.get_params(), self.model_name, os.getenv("GEOPI_OUTPUT_PARAMETERS_PATH"))
        else:
            self.reg_workflow.save_hyper_parameters(self.reg_workflow.automl.best_config, self.model_name, os.getenv("GEOPI_OUTPUT_PARAMETERS_PATH"))

        # Common components for every regression algorithm
        self.reg_workflow.common_components(is_automl)

        # Special components of different algorithms
        self.reg_workflow.special_components(is_automl)

        # Save the prediction result
        self.reg_workflow.data_save(y_train_predict, "Y Train Predict", os.getenv("GEOPI_OUTPUT_ARTIFACTS_DATA_PATH"), MLFLOW_ARTIFACT_DATA_PATH, "Model Train Prediction")
        self.reg_workflow.data_save(y_test_predict, "Y Test Predict", os.getenv("GEOPI_OUTPUT_ARTIFACTS_DATA_PATH"), MLFLOW_ARTIFACT_DATA_PATH, "Model Test Prediction")

        # Save the trained model
        self.reg_workflow.model_save(is_automl)



from typing import Dict

from rich import print

from ....constants import SECTION
from ....data.data_readiness import num_input, str_input


def decision_tree_manual_hyper_parameters() -> Dict:
    """Manually set hyperparameters.

    Returns
    -------
    hype_parameters : dict
    """
    print("Criterion: The function to measure the quality of a split. Supported criteria are 'gini' for the Gini impurity and 'log_loss' and 'entropy' both for the Shannon information gain.")
    print("The default value is 'gini'. Optional criterions are 'entropy' and 'log_loss'.")
    criterions = ["gini", "entropy", "log_loss"]
    criterion = str_input(criterions, SECTION[2])
    print("Max Depth: The maximum depth of the tree. If None, then nodes are expanded until all leaves are pure or until all leaves contain less than min_samples_split samples.")
    print("Please specify the maximum depth of the tree. A good starting range could be between 3 and 15, such as 4.")
    max_depth = num_input(SECTION[2], "@Max Depth: ")
    print("Min Samples Split: The minimum number of samples required to split an internal node.")
    print("Please specify the minimum number of samples required to split an internal node. A good starting range could be between 2 and 10, such as 3.")
    min_samples_split = num_input(SECTION[2], "@Min Samples Split: ")
    print("Min Samples Leaf: The minimum number of samples required to be at a leaf node.")
    print("Please specify the minimum number of samples required to be at a leaf node. A good starting range could be between 1 and 10, such as 2.")
    min_samples_leaf = num_input(SECTION[2], "@Min Samples Leaf: ")
    print("Max Features: The number of features to consider when looking for the best split.")
    print("Please specify the number of features to consider when looking for the best split. A good starting range could be between 1 and the total number of features in the dataset.")
    max_features = num_input(SECTION[2], "@Max Features: ")
    hyper_parameters = {
        "criterion": criterion,
        "max_depth": max_depth,
        "min_samples_split": min_samples_split,
        "min_samples_leaf": min_samples_leaf,
        "max_features": max_features,
    }
    return hyper_parameters



from typing import Dict

from rich import print

from ....constants import SECTION
from ....data.data_readiness import float_input, num_input, str_input


def kmeans_manual_hyper_parameters() -> Dict:
    """Manually set hyperparameters.

    Returns
    -------
    hyper_parameters : dict
    """
    print("N Clusters: The number of clusters to form as well as the number of centroids to generate.")
    print("Please specify the number of clusters for KMeans. A good starting range could be between 2 and 10, such as 4.")
    n_clusters = num_input(SECTION[2], "N Clusters: ")
    print("Init: Method for initialization of centroids. The centroids represent the center points of the clusters in the dataset.")
    print("Please specify the method for initialization of centroids. It is generally recommended to leave it as 'k-means++'.")
    inits = ["k-means++", "random"]
    init = str_input(inits, SECTION[2])
    print("Max Iter: Maximum number of iterations of the k-means algorithm for a single run.")
    print("Please specify the maximum number of iterations of the k-means algorithm for a single run. A good starting range could be between 100 and 500, such as 300.")
    max_iters = num_input(SECTION[2], "Max Iter: ")
    print("Tolerance: Relative tolerance with regards to inertia to declare convergence.")
    print("Please specify the relative tolerance with regards to inertia to declare convergence. A good starting range could be between 0.0001 and 0.001, such as 0.0005.")
    tol = float_input(0.0005, SECTION[2], "Tolerance: ")
    print("Algorithm: The algorithm to use for the computation.")
    print("Please specify the algorithm to use for the computation. It is generally recommended to leave it as 'auto'.")
    print("Auto: selects 'elkan' for dense data and 'full' for sparse data. 'elkan' is generally faster on data with lower dimensionality, while 'full' is faster on data with higher dimensionality")
    algorithms = ["auto", "full", "elkan"]
    algorithm = str_input(algorithms, SECTION[2])
    hyper_parameters = {"n_clusters": n_clusters, "init": init, "max_iter": max_iters, "tol": tol, "algorithm": algorithm}
    return hyper_parameters



# -*- coding: utf-8 -*-
from typing import Dict

from rich import print

from ....constants import SECTION
from ....data.data_readiness import bool_input, float_input, num_input


def extra_trees_manual_hyper_parameters() -> Dict:
    """Manually set hyperparameters.

    Returns
    -------
    hyper_parameters : dict
    """
    print("N Estimators: The number of trees in the forest.")
    print("Please specify the number of trees in the forest. A good starting range could be between 50 and 500, such as 100.")
    print("For small datasets, a range of 50 to 200 trees may be sufficient, while for larger datasets, a range of 200 to 500 trees may be appropriate.")
    n_estimators = num_input(SECTION[2], "@N Estimators: ")
    print("Max Depth: The maximum depth of a tree. Increasing this value will make the model more complex and more likely to overfit.")
    print("Please specify the maximum depth of a tree. A good starting range could be between 5 and 10, such as 4.")
    max_depth = num_input(SECTION[2], "@Max Depth: ")
    print("Min Samples Split: The minimum number of samples required to split an internal node.")
    print("Please specify the minimum number of samples required to split an internal node. A good starting range could be between 2 and 10, such as 2.")
    min_samples_split = num_input(SECTION[2], "@Min Samples Split: ")
    print("Min Samples Leaf: The minimum number of samples required to be at a leaf node.")
    print("Please specify the minimum number of samples required to be at a leaf node. A good starting range could be between 1 and 10, such as 1.")
    min_samples_leaf = num_input(SECTION[2], "@Min Samples Leaf: ")
    print("Max Features: The number of features to consider when looking for the best split.")
    print("Please specify the number of features to consider when looking for the best split. A good starting range could be between 1 and the total number of features in the dataset.")
    print("It's recommended to start with a value of sqrt(n_features) or log2(n_features). Make sure to use integer values.")
    max_features = num_input(SECTION[2], "@Max Features: ")
    print(
        "Bootstrap: Whether bootstrap samples are used when building trees. Bootstrapping is a technique where a random subset of the data is sampled with replacement"
        " to create a new dataset of the same size as the original. This new dataset is then used to construct a decision tree in the ensemble. If False, the whole dataset is used to build each tree."
    )
    print("Please specify whether bootstrap samples are used when building trees. It is generally recommended to leave it as True.")
    bootstrap = bool_input(SECTION[2])
    max_samples = None
    if bootstrap:
        print("Max Samples: The number of samples to draw from X_train to train each base estimator.")
        print("Please specify the ratio. A good starting range could be between 0.5 and 1, such as 0.8.")
        max_samples = float_input(0.8, SECTION[2], "@Max Samples: ")
    print(
        "oob_score: Whether to use out-of-bag samples to estimate the generalization accuracy. When the oob_score hyperparameter is set to True, Extra Trees will use a random subset of the data"
        " to train each decision tree in the ensemble, and the remaining data that was not used for training (the out-of-bag samples) will be used to calculate the OOB score. "
    )
    print("Please specify whether to use out-of-bag samples to estimate the generalization accuracy. It is generally recommended to leave it as True.")
    oob_score = bool_input(SECTION[2])
    hyper_parameters = {
        "n_estimators": n_estimators,
        "max_depth": max_depth,
        "min_samples_split": min_samples_split,
        "min_samples_leaf": min_samples_leaf,
        "max_features": max_features,
        "bootstrap": bootstrap,
        "oob_score": oob_score,
    }
    if not max_samples:
        # Use the default value provided by sklearn.ensemble.ExtraTreesRegressor.
        hyper_parameters["max_samples"] = None
    else:
        hyper_parameters["max_samples"] = max_samples
    return hyper_parameters



# -*- coding: utf-8 -*-
from typing import Dict

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from mpl_toolkits.mplot3d import Axes3D
from rich import print

from ....constants import SECTION
from ....data.data_readiness import bool_input


def linear_regression_manual_hyper_parameters() -> Dict:
    """Manually set hyperparameters.

    Returns
    -------
    hyper_parameters : dict
    """
    print("Fit Intercept: This hyperparameter specifies whether to calculate the intercept (also called the bias term) for this model.")
    print("Please specify whether to calculate the intercept for this model. It is generally recommended to leave it as True.")
    fit_intercept = bool_input(SECTION[2])
    hyper_parameters = {"fit_intercept": fit_intercept}
    return hyper_parameters


def plot_2d_scatter_diagram(feature_data: pd.DataFrame, target_data: pd.DataFrame) -> None:
    """Plot a 2D scatter diagram with the data set below.

    Parameters
    ----------
    feature_data : pd.DataFrame (n_samples, n_components)
        Data selected by user.

    target_data : pd.DataFrame (n_samples, n_components)
        The target values.
    """
    plt.figure(figsize=(14, 10))
    plt.scatter(feature_data.values, target_data.values)
    plt.xlabel(feature_data.columns[0])
    plt.ylabel(target_data.columns[0])
    plt.title("2D Scatter Diagram")
    plt.grid()


def plot_2d_line_diagram(feature_data: pd.DataFrame, target_data: pd.DataFrame, y_test_predict: pd.DataFrame) -> None:
    """Plot a 2D line diagram with the data set below.

    Parameters
    ----------
    feature_data : pd.DataFrame (n_samples, n_components)
        Data selected by user.

    target_data : pd.DataFrame (n_samples, n_components)
        The target values.

    y_test_predict : pd.DataFrame (n_samples, n_components)
        The predicted target values.
    """
    plt.figure(figsize=(14, 10))
    plt.scatter(feature_data.values, target_data.values, label="Data Points")
    plt.plot(feature_data.values, y_test_predict.values, label="Regression Line")
    plt.xlabel(feature_data.columns[0])
    plt.ylabel(target_data.columns[0])
    plt.title("2D Line Diagram")
    plt.legend()
    plt.grid()


def plot_3d_scatter_diagram(feature_data: pd.DataFrame, target_data: pd.DataFrame) -> None:
    """Plot a 3D scatter diagram with the data set below.

    Parameters
    ----------
    feature_data : pd.DataFrame (n_samples, n_components)
        Data selected by user.

    target_data : pd.DataFrame (n_samples, n_components)
        The target values.
    """
    x = feature_data.iloc[:, 0]
    y = feature_data.iloc[:, 1]
    z = target_data
    name_list = feature_data.columns.values.tolist()
    fig = plt.figure(figsize=(14, 10))
    ax = Axes3D(fig)
    ax.scatter(x, y, z)
    ax.set_xlabel(name_list[0], fontdict={"size": 10, "color": "black"})
    ax.set_ylabel(name_list[1], fontdict={"size": 10, "color": "r"})
    ax.set_zlabel(target_data.columns[0], fontdict={"size": 10, "color": "blue"})
    plt.title("3D Scatter Diagram")
    plt.grid()


def plot_3d_surface_diagram(feature_data: pd.DataFrame, target_data: pd.DataFrame, y_test_predict: pd.DataFrame) -> None:
    """Plot a 3D surface diagram with the data set below.

    Parameters
    ----------
    feature_data : pd.DataFrame (n_samples, n_components)
        Data selected by user.

    target_data : pd.DataFrame (n_samples, n_components)
        The target values.

    y_test_predict : pd.DataFrame (n_samples, n_components)
        The predicted target values.
    """
    x = feature_data.iloc[:, 0]
    y = feature_data.iloc[:, 1]
    z = target_data.iloc[:, 0]  # Ensure that z is 1D, not a DataFrame
    name_list = feature_data.columns.values.tolist()
    fig = plt.figure(figsize=(14, 10))
    ax = Axes3D(fig)
    ax.scatter(x, y, z)
    # Create meshgrid for x and y
    x_grid, y_grid = np.meshgrid(np.linspace(x.min(), x.max(), 100), np.linspace(y.min(), y.max(), 100))
    # Interpolate y_test_predict values on the grid
    z_grid = np.interp(np.ravel(x_grid), x, y_test_predict.iloc[:, 0])
    z_grid = z_grid.reshape(x_grid.shape)
    # Plot the 3D surface using the grid
    ax.plot_surface(x_grid, y_grid, z_grid, alpha=0.3, color="red")
    ax.set_xlabel(name_list[0], fontdict={"size": 10, "color": "black"})
    ax.set_ylabel(name_list[1], fontdict={"size": 10, "color": "r"})
    ax.set_zlabel(target_data.columns[0], fontdict={"size": 10, "color": "blue"})
    plt.title("3D Surface Diagram")
    plt.grid()
    plt.legend()



# -*- coding: utf-8 -*-
import re
from typing import Dict, List

import matplotlib.pyplot as plt
import mlflow
import numpy as np
import pandas as pd
from imblearn.over_sampling import RandomOverSampler
from imblearn.pipeline import Pipeline
from imblearn.under_sampling import RandomUnderSampler
from rich import print
from sklearn.metrics import ConfusionMatrixDisplay, accuracy_score, confusion_matrix, f1_score, make_scorer, precision_recall_curve, precision_score, recall_score, roc_curve
from sklearn.model_selection import cross_validate
from sklearn.preprocessing import LabelEncoder

from ....constants import CALCULATION_METHOD_OPTION, SECTION
from ....data.data_readiness import limit_num_input, num2option, num_input


def score(y_true: pd.DataFrame, y_predict: pd.DataFrame) -> tuple[str, Dict]:
    """Calculate the scores of the classification model.

    Parameters
    ----------
    y_true : pd.DataFrame (n_samples, n_components)
        The true target values.

    y_predict : pd.DataFrame (n_samples, n_components)
        The predicted target values.

    Returns
    -------
    average : str
        Metric parameters.

    scores : dict
        The scores of the classification model.
    """
    average = "binary"
    if int(y_true.nunique().values) > 2:
        print("Please select calculation method:")
        print("[bold green]Micro[/bold green]: Calculate metrics globally by counting the total true positives, false negatives and false positives.")
        print("[bold green]Macro[/bold green]: Calculate metrics for each label, and find their unweighted mean. This does not take label imbalance into account.")
        print("[bold green]Weighted[/bold green]: Calculate metrics for each label, and find their average weighted by support (the number of true instances for each label).")
        num2option(CALCULATION_METHOD_OPTION)
        average_num = limit_num_input(CALCULATION_METHOD_OPTION, SECTION[0], num_input)
        if average_num == 1:
            average = "micro"
        elif average_num == 2:
            average = "macro"
        elif average_num == 3:
            average = "weighted"
    accuracy = accuracy_score(y_true, y_predict)
    precision = precision_score(y_true, y_predict, average=average)
    recall = recall_score(y_true, y_predict, average=average)
    f1 = f1_score(y_true, y_predict, average=average)
    print("Accuracy: ", accuracy)
    print("Precision:", precision)
    print("Recall:", recall)
    print("F1 Score:", f1)
    scores = {
        "Accuracy": accuracy,
        "Precision": precision,
        "Recall": recall,
        "F1 Score": f1,
    }
    return average, scores


def plot_confusion_matrix(y_test: pd.DataFrame, y_test_predict: pd.DataFrame, trained_model: object) -> np.ndarray:
    """Plot the confusion matrix.

    Parameters
    ----------
    y_test : pd.DataFrame (n_samples, n_components)
        The testing target values.

    y_test_predict : pd.DataFrame (n_samples, n_components)
        The predicted target values.

    trained_model : sklearn algorithm model
        The sklearn algorithm model trained with X_train data.

    Returns
    -------
    cm : np.ndarray
        The confusion matrix.
    """
    cm = confusion_matrix(y_test, y_test_predict)
    print(cm)
    plt.figure()
    disp = ConfusionMatrixDisplay(confusion_matrix=cm, display_labels=trained_model.classes_)
    disp.plot()
    return cm


def display_cross_validation_scores(scores: np.ndarray, score_name: str) -> Dict:
    """Display the scores of cross-validation.

    Parameters
    ----------
    scores : np.ndarray
        The scores of cross-validation.

    score_name : str
        The name of the score.

    Returns
    -------
    cv_scores : dict
        The scores of cross-validation.
    """
    cv_scores = {
        "Fold Scores": str(scores.tolist()),
        "Mean": scores.mean(),
        "Standard Deviation": scores.std(),
    }
    print("Scores:", cv_scores["Fold Scores"])
    print("Mean:", cv_scores["Mean"])
    print("Standard deviation:", cv_scores["Standard Deviation"])
    mlflow.log_metric(f"CV - {score_name} - Mean", cv_scores["Mean"])
    mlflow.log_metric(f"CV - {score_name} - Standard Deviation", cv_scores["Standard Deviation"])
    return cv_scores


def cross_validation(trained_model: object, X_train: pd.DataFrame, y_train: pd.DataFrame, average: str, cv_num: int = 10) -> Dict:
    """Evaluate metric(s) by cross-validation and also record fit/score times.

    Parameters
    ----------
    trained_model : object
        The model trained.

    X_train : pd.DataFrame (n_samples, n_components)
        The training feature data.

    y_train : pd.DataFrame (n_samples, n_components)
        The training target values.

    average : str
        Metric parameters.

    cv_num : int
        Determines the cross-validation splitting strategy.

    Returns
    -------
    scores_result : dict
        The scores of cross-validation.
    """
    if average == "binary":
        scoring = {
            "accuracy": make_scorer(accuracy_score),
            "precision": make_scorer(precision_score, average="binary"),
            "recall": make_scorer(recall_score, average="binary"),
            "f1": make_scorer(f1_score, average="binary"),
        }
    elif average == "micro":
        scoring = {
            "accuracy": make_scorer(accuracy_score),
            "precision": make_scorer(precision_score, average="micro"),
            "recall": make_scorer(recall_score, average="micro"),
            "f1": make_scorer(f1_score, average="micro"),
        }
    elif average == "macro":
        scoring = {
            "accuracy": make_scorer(accuracy_score),
            "precision": make_scorer(precision_score, average="macro"),
            "recall": make_scorer(recall_score, average="macro"),
            "f1": make_scorer(f1_score, average="macro"),
        }
    elif average == "weighted":
        scoring = {
            "accuracy": make_scorer(accuracy_score),
            "precision": make_scorer(precision_score, average="weighted"),
            "recall": make_scorer(recall_score, average="weighted"),
            "f1": make_scorer(f1_score, average="weighted"),
        }
    scores = cross_validate(trained_model, X_train, y_train, scoring=scoring, cv=cv_num)
    del scores["fit_time"]
    del scores["score_time"]
    # the keys follow the returns of cross_validate in scikit-learn
    scores2display = {
        "test_accuracy": "Accuracy",
        "test_precision": "Precision",
        "test_recall": "Recall",
        "test_f1": "F1 Score",
    }
    scores_result = {"K-Fold": cv_num}
    for key, values in scores.items():
        print("*", scores2display[key], "*")
        cv_scores = display_cross_validation_scores(values, scores2display[key])
        scores_result[scores2display[key]] = cv_scores
        print("-------------")
    return scores_result


def plot_precision_recall(X_test, y_test, trained_model: object, algorithm_name: str) -> tuple:
    """Plot the precision-recall curve.

    Parameters
    ----------
    X_test : pd.DataFrame (n_samples, n_components)
        The testing feature data.

    y_test : pd.DataFrame (n_samples, n_components)
        The testing target values.

    trained_model : object
        The model trained.

    algorithm_name : str
        The name of the algorithm.

    Returns
    -------
    y_probs : np.ndarray
        The probabilities of the model.

    precisions : np.ndarray
        The precision of the model.

    recalls : np.ndarray
        The recall of the model.

    thresholds : np.ndarray
        The thresholds of the model.
    """
    #  Predict probabilities for the positive class
    y_probs = trained_model.predict_proba(X_test)[:, 1]
    precisions, recalls, thresholds = precision_recall_curve(y_test, y_probs)
    plt.figure()
    plt.plot(thresholds, precisions[:-1], "b--", label="Precision")
    plt.plot(thresholds, recalls[:-1], "g-", label="Recall")
    plt.legend(labels=["Precision", "Recall"], loc="best")
    plt.title(f"Precision Recall Curve - {algorithm_name}")
    return y_probs, precisions, recalls, thresholds


def plot_ROC(X_test: pd.DataFrame, y_test: pd.DataFrame, trained_model: object, algorithm_name: str) -> tuple:
    """Plot the ROC curve.

    Parameters
    ----------
    X_test : pd.DataFrame (n_samples, n_components)
        The testing feature data.

    y_test : pd.DataFrame (n_samples, n_components)
        The testing target values.

    trained_model : object
        The model trained.

    algorithm_name : str
        The name of the algorithm.

    Returns
    -------
    y_probs : np.ndarray
        The probabilities of the model.

    fpr : np.ndarray
        The false positive rate of the model.

    tpr : np.ndarray
        The true positive rate of the model.

    thresholds : np.ndarray
        The thresholds of the model.
    """
    y_probs = trained_model.predict_proba(X_test)[:, 1]
    fpr, tpr, thresholds = roc_curve(y_test, y_probs)
    plt.figure()
    plt.plot(fpr, tpr, linewidth=2)
    plt.plot([0, 1], [0, 1], "r--")
    plt.xlabel("False Positive Rate")
    plt.ylabel("True Positive Rate (Recall)")
    plt.title(f"ROC Curve - {algorithm_name}")
    return y_probs, fpr, tpr, thresholds


def plot_2d_decision_boundary(X: pd.DataFrame, X_test: pd.DataFrame, trained_model: object, image_config: Dict) -> None:
    """Plot the decision boundary of the trained model with the testing data set below.

    Parameters
    ----------
    X : pd.DataFrame (n_samples, n_components)
        The complete feature data.

    X_test : pd.DataFrame (n_samples, n_components)
        The testing feature data.

    trained_model : sklearn algorithm model
        The sklearn algorithm model trained with X_train data.

    image_config : dict
        The configuration of the image.
    """

    # Prepare the data for contour plot

    x0s = np.arange(X.iloc[:, 0].min(), X.iloc[:, 0].max(), (X.iloc[:, 0].max() - X.iloc[:, 0].min()) / 50)
    x1s = np.arange(X.iloc[:, 1].min(), X.iloc[:, 1].max(), (X.iloc[:, 1].max() - X.iloc[:, 1].min()) / 50)
    x0, x1 = np.meshgrid(x0s, x1s)
    input_array = np.c_[x0.ravel(), x1.ravel()]
    labels = trained_model.predict(input_array).reshape(x0.shape)

    # Use the code below when the dimensions of X are greater than 2

    # create drawing canvas
    fig, ax = plt.subplots(figsize=(image_config["width"], image_config["height"]), dpi=image_config["dpi"])
    ax.patch.set_facecolor("cornsilk")

    # draw the main content
    ax.contourf(x0, x1, labels, cmap=image_config["cmap2"], alpha=image_config["alpha1"])
    ax.scatter(
        X_test.iloc[:, 0],
        X_test.iloc[:, 1],
        alpha=image_config["alpha2"],
        marker=image_config["marker_angle"],
        cmap=image_config["cmap"],
    )
    # automatically optimize picture layout structure
    fig.tight_layout()
    xmin, xmax = ax.get_xlim()
    ymin, ymax = ax.get_ylim()
    x_adjustment = (xmax - xmin) * 0.01
    y_adjustment = (ymax - ymin) * 0.01
    ax.axis([xmin - x_adjustment, xmax + x_adjustment, ymin - y_adjustment, ymax + y_adjustment])

    # convert the font of the axes
    ax.set_xlabel(X.columns[0])
    ax.set_ylabel(X.columns[1])
    plt.tick_params(labelsize=image_config["labelsize"])  # adjust the font size of the axis label
    # plt.setp(ax.get_xticklabels(), rotation=image_config['xrotation'], ha=image_config['xha'],
    #          rotation_mode="anchor")  # axis label rotation Angle
    # plt.setp(ax.get_yticklabels(), rotation=image_config['rot'], ha=image_config['yha'],
    #          rotation_mode="anchor")  # axis label rotation Angle
    x1_label = ax.get_xticklabels()  # adjust the axis label font
    [x1_label_temp.set_fontname(image_config["axislabelfont"]) for x1_label_temp in x1_label]
    y1_label = ax.get_yticklabels()
    [y1_label_temp.set_fontname(image_config["axislabelfont"]) for y1_label_temp in y1_label]

    ax.set_title(
        label=image_config["title_label"],
        fontdict={
            "size": image_config["title_size"],
            "color": image_config["title_color"],
            "family": image_config["title_font"],
        },
        loc=image_config["title_location"],
        pad=image_config["title_pad"],
    )


def resampler(X_train: pd.DataFrame, y_train: pd.DataFrame, method: List[str], method_idx: int) -> tuple[dict, pd.DataFrame, pd.DataFrame]:
    """Use this method when the classification dataset has an unbalanced number of categories.

    Parameters
    ----------
    X_train : pd.DataFrame (n_samples, n_components)
        The train feature data.

    y_train : pd.DataFrame (n_samples, n_label)
        The train label data.

    method : list
        The specific method of resampling.

    method_idx : int
        The index corresponding to the specific method of resampling.

    Returns
    -------
    sample_balance_config : dict
        The configuration of the resampling method.

    X_train_resampled : pd.DataFrame (n_samples, n_components)
        The resampled train feature data.

    y_train_resampled : pd.DataFrame (n_samples, n_label)
        The resampled train label data.
    """

    if method[method_idx] == "Over Sampling":
        resampler = RandomOverSampler()
        X_train_resampled, y_train_resampled = resampler.fit_resample(X_train, y_train)
        sample_balance_config = {type(resampler).__name__: resampler.get_params()}
    elif method[method_idx] == "Under Sampling":
        resampler = RandomUnderSampler()
        X_train_resampled, y_train_resampled = resampler.fit_resample(X_train, y_train)
        sample_balance_config = {type(resampler).__name__: resampler.get_params()}
    elif method[method_idx] == "Oversampling and Undersampling":
        over = RandomOverSampler()
        under = RandomUnderSampler()
        over_under_steps = [("oversample", over), ("undersample", under)]
        resampler = Pipeline(steps=over_under_steps)
        X_train_resampled, y_train_resampled = resampler.fit_resample(X_train, y_train)
        sample_balance_config = {type(resampler[0]).__name__: resampler[0].get_params(), type(resampler[1]).__name__: resampler[1].get_params()}
    return sample_balance_config, X_train_resampled, y_train_resampled


def reset_label(y: pd.DataFrame, y_train: pd.DataFrame, y_test: pd.DataFrame, method: List[str], method_idx: int) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Reset label.

    Parameters
    ----------
    y : pd.DataFrame
        The label data set.

    y_train : pd.DataFrame
        The label data on train set.

    y_test : pd.DataFrame
        The label data on test set.

    method : list
        The specific method of customizing label.

    method_idx : int
        The index corresponding to the specific method of customizing label.

    Returns
    -------
    y_reset : pd.DataFrame
        The label data set after reseting label.

    y_train_reset : pd.DataFrame
        The label data on train set after reseting label.

    y_test_reset : pd.DataFrame
        The label data on test set after reseting label.
    """
    y_colunm_name = list(y)[0]
    label_range = int(y.nunique().values)
    print("----Originla label----\n", y.drop_duplicates().reset_index(drop=True))
    if method[method_idx] == "Automatic Coding":
        label_encoder = LabelEncoder()
        label_encoder.fit(y.squeeze())
        y_reset = label_encoder.transform(y.squeeze())
        y_reset = pd.DataFrame({y_colunm_name: y_reset})
        y_train_reset = label_encoder.transform(y_train.squeeze())
        y_train_reset = pd.DataFrame({y_colunm_name: y_train_reset})
        y_test_reset = label_encoder.transform(y_test.squeeze())
        y_test_reset = pd.DataFrame({y_colunm_name: y_test_reset})
    elif method[method_idx] == "Custom Numeric Labels":
        print("When doing customization, the new label should start from 0.")
        print("Case 1: For binary classification, the range of the new label must be from 0 to 1.")
        print("For example, if your original labels are 2 and 4, you can replace them with 0 and 1 respectively.")
        print("Input format: [bold green][original label,[/bold green] [bold green]new label][/bold green], such as [bold green][2, 0]; [4, 1][/bold green].")
        print("This means you want to replace 2 with 0 and 4 with 1.")
        print(f"Case 2: For multi-classification, the range of the new label must be from 0 to {label_range}(the number of category - 1).")
        print("For example, if your original labels are 2, 4 and 6, you can replace them with 0, 1 and 2 respectively.")
        print("Input format: [bold green][original label,[/bold green] [bold green]new label][/bold green], such as [bold green][2, 0]; [4, 1]; [6, 2][/bold green].")
        print("This means you want to replace 2 with 0, 4 with 1 and 6 with 2.")
        original_new_label = input("Select the label range you want to process.\n" "@input: ")
        label_sequence = re.findall(r"\[(\d+)\,\s*(\d+)\]", original_new_label)
        customize_label_mapping = {int(label): int(value) for label, value in label_sequence}
        y_reset = y.squeeze().map(customize_label_mapping)
        y_reset = pd.DataFrame({y_colunm_name: y_reset})
        y_train_reset = y_train.squeeze().map(customize_label_mapping)
        y_train_reset = pd.DataFrame({y_colunm_name: y_train_reset})
        y_test_reset = y_test.squeeze().map(customize_label_mapping)
        y_test_reset = pd.DataFrame({y_colunm_name: y_test_reset})
    elif method[method_idx] == "Custom Non-numeric Labels":
        print("When doing customization, the new label should start from 0.")
        print("Case 1: For binary classification, the range of the new label must be from 0 to 1.")
        print("For example, if your original labels are 'A' and 'B', you can replace them with 0 and 1 respectively.")
        print("Input format: [bold green]['original label', new label][/bold green], such as [bold green]['A', 0]; ['B', 1][/bold green].")
        print("This means you want to replace A with 0 and B with 1.")
        print(f"Case 2: For multi-classification, the range of the new label must be from 0 to {label_range}(the number of category - 1).")
        print("For example, if your original labels are A, B and C, you can replace them with 0, 1 and 2 respectively.")
        print("Input format: [bold green]['original label', new label][/bold green], such as [bold green]['A', 2]; ['B', 0]; ['C', 1][/bold green].")
        print("This means you want to replace A with 2, B with 0 and C with 1.")
        original_new_label = input("Select the label range you want to process.\n" "@input: ")
        label_sequence = re.findall(r"\[\'(.*?)\',\s*(\d+)\]", original_new_label)
        customize_label_mapping = {label: int(value) for label, value in label_sequence}
        y_reset = y.squeeze().map(customize_label_mapping)
        y_reset = pd.DataFrame({y_colunm_name: y_reset})
        y_train_reset = y_train.squeeze().map(customize_label_mapping)
        y_train_reset = pd.DataFrame({y_colunm_name: y_train_reset})
        y_test_reset = y_test.squeeze().map(customize_label_mapping)
        y_test_reset = pd.DataFrame({y_colunm_name: y_test_reset})

    return y_reset, y_train_reset, y_test_reset
